;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x998e64e33658d0bb52ff9307a4881336 . "./code.cg2js/cg-acache.fasl")
(#x64b41c0d7efa51153cd7b77a7a548a45 . "./code.cg2js/cg-base.fasl")
(#xbf0481cf5ec46a9871d8169badd99e0a . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x784f37de6f9156de250e442a8786f85c . "./code.cg2js/cg-bitmap-pane.fasl")
(#x817a836e21ca38271d3f5944ca05eda7 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xb62ff1b8a36b7245c81241fb03b0dc02 . "./code.cg2js/cg-button.fasl")
(#x069c21fc09446753aa03a925bfaee813 . "./code.cg2js/cg-calendar.fasl")
(#x7bb561aaa9fb5599657c8e2dd832d655 . "./code.cg2js/cg-caret.fasl")
(#x9f68995a3735aff12ef53782b35d6deb . "./code.cg2js/cg-chart-or-plot.fasl")
(#xdd593280f2b50cb88ae54b63db2fd09f . "./code.cg2js/cg-chart-widget.fasl")
(#x6d38c5c3f7632ed895d11e54e158b745 . "./code.cg2js/cg-check-box.fasl")
(#xd1204f9478539ac1701297a6319da496 . "./code.cg2js/cg-choice-list.fasl")
(#xc21e2452fcdc590aa44d93347b43224f . "./code.cg2js/cg-choose-printer.fasl")
(#x8a7219280996c262d93b984c96767914 . "./code.cg2js/cg-class-grid.fasl")
(#xf36687e09765c441d3fbc64063825a67 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x0d5af7b2a0d83198d22dbf3d1efc7d86 . "./code.cg2js/cg-class-support.fasl")
(#x708f6f20aca5aaf3a08354a37106c245 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x089d0e755ff4f858e1a24ce0265c6ad5 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x8ceda6019624853b32cc96d094bc93b7 . "./code.cg2js/cg-clipboard.fasl")
(#x6cb3e0d126b8cf4ab1b6f233a51b227f . "./code.cg2js/cg-color-dialog.fasl")
(#x2db4057df90119db63e3e4fc39dbe926 . "./code.cg2js/cg-combo-box.fasl")
(#xcbd911497d5b8cc2691527482f750e11 . "./code.cg2js/cg-common-control.fasl")
(#xdad532e3357ff7b90ae9efa14dd97305 . "./code.cg2js/cg-comtab.fasl")
(#x069ca26561bb465fd2a53925886cf83c . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xc790fbe4edc1706c0007a672e2f81ea3 . "./code.cg2js/cg-curve.fasl")
(#x17e688a4fb02cd6328c3bfd67d2ad4bb . "./code.cg2js/cg-dde-utils.fasl")
(#x629b69dc5e3dbfce3abcba7549b9cac0 . "./code.cg2js/cg-dev.fasl")
(#xbead878ee7d0078711948009b8d83ac9 . "./code.cg2js/cg-dialog-item.fasl")
(#xd45acba30fa54d7fc02ce5c70eb1b33b . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x2eeab13eca42322c3d29a19bad6233d9 . "./code.cg2js/cg-directory-dialog.fasl")
(#x60f5631a9ca8449c83379e5cf4999ce2 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xbfa120719cd1df2cae18a32ac712c455 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x0662e3e84b0ad7a4839d61bfb780a5f1 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xaac5439633dae816f3953a5cdd653ec1 . "./code.cg2js/cg-drawable.fasl")
(#xfd825264a0ea6a5ecb61ab77c32b7503 . "./code.cg2js/cg-dropping-outline.fasl")
(#x8eb43c235ef36eab927f00cf65ffd89c . "./code.cg2js/cg-edit-in-place.fasl")
(#xf4901c9869ec2184b5471c71ad9a2af4 . "./code.cg2js/cg-editable-text.fasl")
(#x095dfcdcd2ea17bd8722377f6425d4b6 . "./code.cg2js/cg-file-dialog.fasl")
(#x999005433ce6e8b4c46bab230290a986 . "./code.cg2js/cg-fill-texture.fasl")
(#x1e24625e7fce9cce67609cc9d4f86b8e . "./code.cg2js/cg-find-string-dialog.fasl")
(#x54a04a586f146e53a5ede8b8ce80daee . "./code.cg2js/cg-font-dialog.fasl")
(#x89ee44dd03cbf9b2c70f042639d7427c . "./code.cg2js/cg-gesture-emulation.fasl")
(#x6028a5aa2d1b5230e44cd16135c94a9e . "./code.cg2js/cg-get-pixmap.fasl")
(#x1c22096dd4254c328404c4ffdb442236 . "./code.cg2js/cg-get-position.fasl")
(#x7f88b5faf60da6869057db4d2c9422ed . "./code.cg2js/cg-graphics-context.fasl")
(#x4869d34d397983c35a6e0c95fce66b25 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xaac9cdaa82a8150412e87561ddbd7c31 . "./code.cg2js/cg-grid-widget.fasl")
(#x8522360e2ed607d989e95423de60dcc2 . "./code.cg2js/cg-group-box.fasl")
(#x5d504086b1daf446f1983c68fb14fa5e . "./code.cg2js/cg-header-control.fasl")
(#x3fb4f0323d9c90352a86485f9ac1a6d7 . "./code.cg2js/cg-hotspot.fasl")
(#x3840f74cd80c22966937239e6d808961 . "./code.cg2js/cg-html-dialog.fasl")
(#xa499d1765974c1c694423676c457c0db . "./code.cg2js/cg-html-widget.fasl")
(#xf7bb28c7469e529084488de9e6a791ea . "./code.cg2js/cg-icon-pixmap.fasl")
(#x1df6874dd671a6d27697ccc779faefa4 . "./code.cg2js/cg-icon.fasl")
(#xd9c808270ce0e9eb2110657139c71ebe . "./code.cg2js/cg-ie.fasl")
(#x76f74e41077056579d132aedb6010fcc . "./code.cg2js/cg-intersect-posbox.fasl")
(#x883cfeb77c4fde5575e24216cf21ef67 . "./code.cg2js/cg-intersect.fasl")
(#xa4b8d5ae12d57191590059ebac96772d . "./code.cg2js/cg-item-list.fasl")
(#x0fc8f9c4fe2ad65de21157c78fba982e . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xcb05ad46da81cef1974749eeb51100f9 . "./code.cg2js/cg-lamp.fasl")
(#xbd6ffce330d612c087d5d5566892e3b2 . "./code.cg2js/cg-layout.fasl")
(#x0c5e9dd3a9570aeb4a0fa74e6f051361 . "./code.cg2js/cg-lettered-menu.fasl")
(#x231e1363980ace7d1aa47287ac1a6b91 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x429e90f42a52d1c04d1c2006771c8260 . "./code.cg2js/cg-lisp-text.fasl")
(#x3f9b373936a605a5c90764aa0e150ccc . "./code.cg2js/cg-lisp-widget.fasl")
(#xb3ab2445460b3f4d4986e0c11deb6f1b . "./code.cg2js/cg-list-view.fasl")
(#xddaa78fdae435b29242eec7ade6c1c92 . "./code.cg2js/cg-mci.fasl")
(#x978ef2c8386797a611e3c5e53ad20368 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x9a934b40359cbf218341d314dab69305 . "./code.cg2js/cg-menu.fasl")
(#x0e0618b8428b53d7abf73007e7577e86 . "./code.cg2js/cg-message-dialog.fasl")
(#x8445259a2129821dc845da144322e8a7 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x6919c654120284e3ca7aef98a0bacf48 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x6df2789e073000492fcbcda94e9acd5c . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xd08944d20fd13bc40905f31ba4003592 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x90f9b224942da863b24432ac0fbdadb9 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x62861e3a65620853311329e4c9bb6c1f . "./code.cg2js/cg-nodes.fasl")
(#x8ab4cdcc8891540683ea1b4f2c1d0019 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x62e1c577c8ed815557e0d38d571e0c29 . "./code.cg2js/cg-object-editor.fasl")
(#x1d5e7b6a41b1af3d720ade148bd030f0 . "./code.cg2js/cg-ocx.fasl")
(#xaa9e0676cdf10b55fe63c7db27e5bb2d . "./code.cg2js/cg-os-widget.fasl")
(#x15e2fb173c41ce927a47cc7533d4cd50 . "./code.cg2js/cg-os-window.fasl")
(#xb59afca7776d6c17f37e54a44c53e7ab . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x3a65ae862f792b04059b3c03f46187de . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xaebd0a27ff430e45b7dfa76707644a19 . "./code.cg2js/cg-outline.fasl")
(#xa2352f10f591333af7d76bc0f91057fe . "./code.cg2js/cg-palette.fasl")
(#x9fc36219e67732dfddc1c5dcbceed756 . "./code.cg2js/cg-paren-matching.fasl")
(#x57d1ee06da06b20ef2e9ea3a4f21da24 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x8db1513bea80a5dc1c07bc24a5499104 . "./code.cg2js/cg-picture-widget.fasl")
(#x929c45a3a5c69fb743e38e402f74c212 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xb575f63529c422d29147a10f6da76575 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x186a902f73aab124e2fe39fdcbd5f7c6 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x4b90fc17bbcbcb0d31022928e3004dbd . "./code.cg2js/cg-pixmap-widget.fasl")
(#xd9bfd779c1775e46eb81155d7a5acbc5 . "./code.cg2js/cg-pixmap.fasl")
(#xdcb16265c2830f1fcd265ec89d1921ce . "./code.cg2js/cg-player.fasl")
(#xcfbcc904693fc00e2e7a57b311b6d1eb . "./code.cg2js/cg-plot-widget.fasl")
(#x6754d1f165c7a3a0c00b3e6fc7a8f5a1 . "./code.cg2js/cg-printing.fasl")
(#xa31fca4f6c151d22ca8294b9fa20bac8 . "./code.cg2js/cg-progress-indicator.fasl")
(#xd6d02128ced9d24292231c87e747bade . "./code.cg2js/cg-project-window.fasl")
(#x026b0d76e06927e9f167b89cccf1d5fd . "./code.cg2js/cg-property.fasl")
(#x6fc3b0a352e231b06ac1aa9ac3a2d7b8 . "./code.cg2js/cg-radio-button.fasl")
(#x8aec459418d664ffcc344519d54644f3 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x90ec70910106eed5d6ea305e1b635900 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x9aa88f5716ed2d0b03273d1104c76bfb . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x44f9297cb758459249c002b88931eb3f . "./code.cg2js/cg-rich-edit.fasl")
(#xe10ee0d36703c7498050db2e94d04010 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x218eeb77e49ac189e4a8243a291448ee . "./code.cg2js/cg-scaling-stream.fasl")
(#x793426e6d06644bde1db49bde53edb02 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x049779cca6fa390d282e1cf5decc7d48 . "./code.cg2js/cg-scroll-bar.fasl")
(#x4a7910f38c7aa01fdd7fcad46ec82a12 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xa7472f6960a10bd28df65a9fdfa6a00c . "./code.cg2js/cg-selected-object.fasl")
(#xabce90d5c43e8e357e64dc5506cc65fa . "./code.cg2js/cg-shortcut-menu.fasl")
(#x36c4cc8238a041f6101e7dc06f1629cf . "./code.cg2js/cg-split-bar.fasl")
(#xcbd84c404e26526a500f178a869b52f3 . "./code.cg2js/cg-static-text.fasl")
(#x22d349e849510e1e0edd23e48c5661a4 . "./code.cg2js/cg-status-bar.fasl")
(#xe7dd71b3838c2ba807e2fa62e2abd5d0 . "./code.cg2js/cg-string-dialog.fasl")
(#xa39625ec93ff663817f59368ee05a8b3 . "./code.cg2js/cg-tab-control.fasl")
(#x6ba3ba4aaf72759208aa141135e2baed . "./code.cg2js/cg-template-string.fasl")
(#xf4da82389a887ee11ad9d0ae7ab7eda9 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x0c8522b35e43e840d22b863efbb9aff7 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x0e36067ebbc22d74195aadbff899b8c5 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xc432bbe92ccef5d6fa47776de7112196 . "./code.cg2js/cg-text-or-combo.fasl")
(#xab2a900ecfc290161441bf0789db5646 . "./code.cg2js/cg-text-widget.fasl")
(#xf19bcc1701cd548120473b41f68df944 . "./code.cg2js/cg-timer.fasl")
(#xbadcc9d129c6ca71b03151f11b3004b3 . "./code.cg2js/cg-toggling-widget.fasl")
(#x73f67e4e09cb6f425165d94228f22b26 . "./code.cg2js/cg-toolbar.fasl")
(#x84d9ab917c02f543d8e35802aaa1cd0f . "./code.cg2js/cg-tooltip.fasl")
(#x2c66d3f38112ceefd7df3141e65cf288 . "./code.cg2js/cg-touch.fasl")
(#x1142cce601ff0eaa5863854b0df992b8 . "./code.cg2js/cg-trackbar.fasl")
(#xd88a5e0d2b30349b8b1cc57454039fc2 . "./code.cg2js/cg-tray.fasl")
(#x81c88b3f7f41df2bab7b82fab37b776f . "./code.cg2js/cg-up-down-control.fasl")
(#x2b2422cf99b18dc1cd57f533a660a74e . "./code.cg2js/cg-utility-dialog.fasl")
(#x86da845f7a28f7002c0feaed9a3a3bc1 . "./code.cg2js/cg-web-browser-dde.fasl")
(#x7f24c725f54468c9fb1ffc9e6b61efa9 . "./code.cg2js/cg-web-browser.fasl")
(#xc3b9f985616059edfea3e3f4c8311d17 . "./code.cg2js/cg-wrap-string.fasl")
(#x22aca60595485c835af496e8f29b5b20 . "./code.cg2js/cg-yes-no-list.fasl")
(#x51c2d37bdfb6a452b6c3cf7720ef8d21 . "./code.cg2js/cg.fasl")
(#xf8e70ea6bb6acc70c04306dbb1ed4535 . "./code.cg2js/cg-yes-no-string.fasl")
(#x1cacd00fd10101dec4587df6c1ff8778 . "./code.idejs/ide-acache.fasl")
(#xa6578b4941e7aef5d9f6cc4f66674ea0 . "./code.idejs/ide-apropos.fasl")
(#x0179692a13875f803d0529286f10e538 . "./code.idejs/ide-base.fasl")
(#x0d71a2904918b39ba895a8856d67faee . "./code.idejs/ide-class-browser.fasl")
(#xb10106e68caaff1b1b9f3615523580d3 . "./code.idejs/ide-class-grapher.fasl")
(#x34464feb450089c7c7f53696a33ae50f . "./code.idejs/ide-clipboard.fasl")
(#xf80edfd96d98b1aa5fc4bad12b9d405a . "./code.idejs/ide-dev.fasl")
(#xc7334fa87ab7a39ed6f12bec5fe23945 . "./code.idejs/ide-editor.fasl")
(#x98802c28641107106cb1a5e83a5f036b . "./code.idejs/ide-find-definitions.fasl")
(#x329aeb55c3daa8db8a4ffd96c275721c . "./code.idejs/ide-find-in-files.fasl")
(#x27da18f315dee2c293382c8040abfece . "./code.idejs/ide-form.fasl")
(#xf46011ed1596a44c5f70d8eb70a1a29a . "./code.idejs/ide-grapher.fasl")
(#xd40d0eee5d0992509610f1578ce36677 . "./code.idejs/ide-inspector.fasl")
(#x058eae149b85d89855ccb70e2cd7df94 . "./code.idejs/ide-intro.fasl")
(#xcdd349a266a9961416ad5ad6cc80acf7 . "./code.idejs/ide-keyshort.fasl")
(#xaf1548377ef4376d02a354698e8c4d2b . "./code.idejs/ide-navigator.fasl")
(#xa5ad31157bb764b2befbe2000dc41ca7 . "./code.idejs/ide-package-list.fasl")
(#x5f0ac8d17f154941cd30d7f2b09fc47a . "./code.idejs/ide-patch-dialog.fasl")
(#x0243ad769c8f68ae804542d679155c1e . "./code.idejs/ide-process-browser.fasl")
(#x92115a3c14de4faaa7e26083bddf0e47 . "./code.idejs/ide-project.fasl")
(#x6283e7c7428c0cd2a0d03b89c05dbe0d . "./code.idejs/ide-trace.fasl")
(#x3c40d27ac63a1af8678b021d22e5de91 . "./code.idejs/ide-window-list.fasl")
(#x953e8636ff3069cca30848f7a6da2b48 . "./code.idejs/ide.fasl")
(#xf82fd41d79afb1da19d111da8345005f . "./cgjs.html")
(#x55fea08d92e113f5a8381c271febbc9e . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
